import type {GetStaticProps, NextPage} from 'next';
import Link from 'next/link';
import * as fs from 'fs/promises';
import * as path from 'path';
import { Layouts, Widgets } from '@tesseract-design/viewfinder-react';
import * as React from 'react'
import {useRouter} from 'next/router';
import {Brand} from '@/components/Brand';
import ReactMarkdown from 'react-markdown';

type Page = {
  id: string,
  href: string,
  label: string,
}

type Props = {
  componentPages: Page[],
  examplePages: Page[],
}

const createSidebarPageLink = (p: Page) => (
  <div
    key={p.id}
  >
    <Link
      href={p.href}
      className="no-underline font-semibold m-0 p-0 leading-none h-8 flex items-center cursor-pointer uppercase text-sm"
    >
      <Layouts.LeftSidebar.SidebarContentContainer>
        {p.label}
      </Layouts.LeftSidebar.SidebarContentContainer>
    </Link>
  </div>
);

const IndexPage: NextPage<Props> = ({
  componentPages,
  examplePages,
}) => {
  const router = useRouter();

  return (
    <Layouts.LeftSidebar.Root
      topBarWidget={
        <Widgets.TopBar
          span="wide"
          brand={
            <Link
              href="/"
              className="no-underline block"
            >
              <Brand />
            </Link>
          }
          menuLink={
            <Link
              href={{
                query: {
                  open: 'sidebar',
                },
              }}
              className="no-underline p-0 m-0"
            >
              <svg
                className="w-6 h-6 fill-none stroke-current stroke-2 linejoin-round linecap-round"
                viewBox="0 0 24 24"
                role="presentation"
              >
                <line x1="3" y1="12" x2="21" y2="12"/>
                <line x1="3" y1="6" x2="21" y2="6"/>
                <line x1="3" y1="18" x2="21" y2="18"/>
              </svg>
            </Link>
          }
        />
      }
      sidebarBaseWidget={
        <Widgets.LeftSidebarBase
          open={router.query.open === 'sidebar'}
        >
          <div className="min-h-full py-4 flex flex-col gap-8">
            <div>
              <nav>
                <h1>
                  <Layouts.LeftSidebar.SidebarContentContainer>
                    Categories
                  </Layouts.LeftSidebar.SidebarContentContainer>
                </h1>
                <div className="flex flex-col gap-2 my-8">
                  {componentPages.map(createSidebarPageLink)}
                </div>
              </nav>
              <nav>
                <h1>
                  <Layouts.LeftSidebar.SidebarContentContainer>
                    Examples
                  </Layouts.LeftSidebar.SidebarContentContainer>
                </h1>
                <div className="flex flex-col gap-4 my-4">
                  {examplePages.map(createSidebarPageLink)}
                </div>
              </nav>
            </div>
            <div className="mb-8">
              <Layouts.LeftSidebar.SidebarContentContainer>
                <div className="flex gap-8">
                  <Link
                    href="https://code.modal.sh/tesseract-design/tesseract"
                    className="no-underline font-semibold m-0 p-0 leading-none h-8 flex items-center cursor-pointer uppercase text-sm"
                  >
                    Repo
                  </Link>
                  <Link
                    href="https://code.modal.sh/tesseract-design/tesseract"
                    className="no-underline font-semibold m-0 p-0 leading-none h-8 flex items-center cursor-pointer uppercase text-sm"
                  >
                    Website
                  </Link>
                  <Link
                    href="https://code.modal.sh/tesseract-design/tesseract/issues"
                    className="no-underline font-semibold m-0 p-0 leading-none h-8 flex items-center cursor-pointer uppercase text-sm"
                  >
                    Issues
                  </Link>
                </div>
              </Layouts.LeftSidebar.SidebarContentContainer>
            </div>
          </div>
        </Widgets.LeftSidebarBase>
      }
    >
      <Layouts.LeftSidebar.MainContentContainer>
        <ReactMarkdown
          className="my-12 leading-loose"
          components={{
            ul: ({node, ordered: _ordered, ...props}) => (
              <ul
                {...props}
                className="list-disc pl-4"
              />
            ),
            li: ({node, ...props}) => (
              <li
                {...props}
                className="list-item pl-4"
              />
            ),
          }}
        >
          {`# Tesseract

## Design
- Tesseract components follow a strict and simple design philosophy.
- The aesthetics of the elements are rigid--any element which users can
  interact with should be easily recognizable--they are determined by the
  usage of the chosen accent color.
- The look and feel of the elements are designed to be less opinionated
  as possible.

## Engineering
- Every component is a derivative of an existing HTML element,
  thus any Tesseract component can substitute for any HTML element
  with added capabilities like improved user experience and consistency.
- In case of overloaded HTML elements such as \`<input>\`, Tesseract
  dissects the element into multiple components each having a single
  defined behavior.
- Tesseract components gracefully degrade to their native counterparts,
  allowing user agents without client-side JavaScript to still use the
  components with minimal loss of functionality.
- Tesseract components render Web-compliant HTML and CSS, thus
  making it resilient to future changes in the HTML/CSS specifications.
- Tesseract components are designed to be used with popular JavaScript
  frameworks like React. Consumption of the components using the framework
  of choice should be seamless, even with the use of form-handling libraries.
- Terminal-based browsers such as Lynx are used to test the components
  for accessibility and graceful degradation.

## Accessibility
- Components are made to be accessible according to Web standards (WAI-ARIA).
- Focus handling is managed sensibly to facilitate functional
  keyboard navigation.
- In cases where there are improvements in user experience, some
  elements may have different markup and appearance. Tesseract
  ensures components are still usable on most assistive technologies given
  both versions of the user interface.
- All components are named in order to depict the purpose, the nature of data
  being operated, and UX expectations.
`}
        </ReactMarkdown>
      </Layouts.LeftSidebar.MainContentContainer>
    </Layouts.LeftSidebar.Root>
  )
}

export const getStaticProps: GetStaticProps = async () => {
  const pagesPath = path.resolve('src/pages');
  const categoriesPath = path.resolve(pagesPath, 'categories');
  const categoriesRaw = await fs.readdir(categoriesPath);
  const categoriesIndexPage = await Promise.all(
    categoriesRaw.map(async (c) => {
      const indexPath = await path.resolve(categoriesPath, c, 'index.tsx');
      try {
        const statResult = await fs.stat(indexPath);
        return [c, statResult.isFile()];
      } catch {
        // noop
      }

      return [c, false];
    })
  ) as [string, boolean][];
  const categories = categoriesIndexPage
    .filter(([, hasIndexPage]) => hasIndexPage)
    .map(([key]) => key);

  const examplesPath = path.resolve(pagesPath, 'examples');
  const examplesRaw = await fs.readdir(examplesPath);
  const examplesIndexPage = await Promise.all(
    examplesRaw.map(async (c) => {
      const indexPath = await path.resolve(examplesPath, c, 'index.tsx');
      try {
        const statResult = await fs.stat(indexPath);
        return [c, statResult.isFile()];
      } catch {
        // noop
      }

      return [c, false];
    })
  ) as [string, boolean][];
  const examples = examplesIndexPage
    .filter(([, hasIndexPage]) => hasIndexPage)
    .map(([key]) => key);

  return {
    props: {
      componentPages: categories.map((c) => ({
        id: c,
        href: `/categories/${c}`,
        label: c.split('-').map((cc) => cc.slice(0, 1).toUpperCase() + cc.slice(1)).join(' '),
      })),
      examplePages: examples.map((e) => ({
        id: e,
        href: `/examples/${e}`,
        label: e.split('-').map((ee) => ee.slice(0, 1).toUpperCase() + ee.slice(1)).join(' '),
      })),
    },
  };
};

export default IndexPage
